@@ -14,6 +14,11 @@ from mspypeline.file_reader import MissingFilesException, BaseReader
 
 
 class MSPInitializer:
+    """
+    | An initializer class which is responsible for creating the directory to save the default YAML configuration
+      file as well as reading and saving the specified settings.
+    | The initializer also operates as a means of passing stored configurations to the plotter classes.
+    """
     # set all file names that are required
     yml_file_name_tmp = "config_tmp.yml"
     yml_file_name = "config.yml"
@@ -25,16 +30,27 @@ class MSPInitializer:
     possible_pathways = sorted([x for x in os.listdir(os.path.join(path_package_config, pathway_path))
                                 if x.endswith(".txt")])
 
-    def __init__(self, dir_: str, file_path_yml: Optional[str] = None, loglevel=logging.DEBUG):
+    def __init__(self, path: str, file_path_yml: Optional[str] = None, loglevel=logging.DEBUG):
+        """
+        Parameters
+        ----------
+        path
+            location where the directory/txt folder to the data can be found.
+        file_path_yml
+            path to the yaml config file
+        loglevel
+            level of the logger
+        """
         self.logger = get_logger(self.__class__.__name__, loglevel=loglevel)
         # create a yaml file reader
         self.yaml = YAML()
-        # self.yaml.indent(mapping=2, sequence=4, offset=2)
-        self.yaml.indent(offset=2)
+        self.yaml.indent(mapping=2, sequence=4, offset=2)
+        # self.yaml.indent(offset=2)
         self.yaml.default_flow_style = False
         self.yaml.width = 4096
 
         # attributes that change upon changing the starting dir
+        #: configurations for the run. also saved configurations for the reader under the respective reader name
         self.configs = {}
         self.reader_data = {}
 
@@ -44,8 +60,12 @@ class MSPInitializer:
         self._start_dir = None
         self._file_path_yaml = None
 
+        # list to store all selectable terms; custom and provided
+        self.list_full_gos = []
+        self.list_full_pathways = []
+
         # set the specified dirs
-        self.start_dir = dir_
+        self.start_dir = path
         if file_path_yml is not None:
             self.file_path_yaml = file_path_yml
 
@@ -67,6 +87,21 @@ class MSPInitializer:
         self.configs = {}
         self.reader_data = {}
         self.file_path_yaml = "file"
+        # see if any custom lists can be found
+        self.list_full_gos = []
+        try:
+            self.list_full_gos += [x for x in os.listdir(os.path.join(self._start_dir, "go_terms"))
+                                   if os.path.isfile(os.path.join(self._start_dir, "go_terms", x))]
+        except FileNotFoundError:
+            pass
+        self.list_full_gos += MSPInitializer.possible_gos
+        self.list_full_pathways = []
+        try:
+            self.list_full_pathways += [x for x in os.listdir(os.path.join(self._start_dir, "pathways"))
+                                        if os.path.isfile(os.path.join(self._start_dir, "pathways", x))]
+        except FileNotFoundError:
+            pass
+        self.list_full_pathways += MSPInitializer.possible_pathways
 
     @property
     def path_config(self):
@@ -74,19 +109,16 @@ class MSPInitializer:
 
     @property
     def file_path_yaml(self):
-        return self._file_path_yaml
-
-    @file_path_yaml.setter
-    def file_path_yaml(self, file_path_yml: str):
         """
+        Setting the yaml file path will set the configurations of the class to the ones specified in the file.
 
-        Parameters
-        ----------
-        file_path_yml
-            can be either:
-                - "default"
-                - "file"
-                - a path to a yml file
+        Note
+        -----
+        The value can be set to either:
+
+        - "default"
+        - "file"
+        - a path to a yml file
 
         Raises
         ------
@@ -96,6 +128,10 @@ class MSPInitializer:
             if the file specified by the file_path_yml was not found
 
         """
+        return self._file_path_yaml
+
+    @file_path_yaml.setter
+    def file_path_yaml(self, file_path_yml: str):
         if file_path_yml.lower() == "default":
             self._file_path_yaml = self.get_default_yml_path()
         elif file_path_yml.lower() == "file":
@@ -117,7 +153,9 @@ class MSPInitializer:
 
     def init_config(self):
         """
-        Creates the directory to save the configuration file and saves the configuration
+        | Creates the directory to save the configuration file if not present, updates and saves the configuration.
+        | The function is usually applied to ensure that configs are provided to the initializer in order to avoid
+          problems initializing the file reader with :meth:`read_data`.
         """
         os.makedirs(self.path_config, exist_ok=True)
         self.update_config_file()
@@ -134,7 +172,7 @@ class MSPInitializer:
         return False
 
     def get_default_yml_path(self) -> str:
-        self.logger.debug("Loading default yml file from: %s, since no (valid) file was selected",
+        self.logger.debug("Loading default yml file from: %s, since 'default' or no (valid) file was selected",
                           path_package)
         return os.path.join(path_package_config, MSPInitializer.default_yml_name)
 
@@ -142,31 +180,25 @@ class MSPInitializer:
         dict_pathway = {}
         dict_go = {}
         for pathway in self.configs.get("pathways"):
-            name, proteins = self.read_config_txt_file(MSPInitializer.pathway_path, pathway)
+            name, proteins = self.read_config_txt_file(pathway)
             dict_pathway[name] = proteins
 
         for go in self.configs.get("go_terms"):
-            name, proteins = self.read_config_txt_file(MSPInitializer.go_path, go)
+            name, proteins = self.read_config_txt_file(go, False)
             dict_go[name] = proteins
         return dict_pathway, dict_go
 
-    def read_config_txt_file(self, path, file) -> Tuple[str, list]:
-        fullpath = os.path.join(path_package_config, path, file)
-        if path == MSPInitializer.pathway_path:
-            with open(fullpath) as f:
-                name = f.readline().strip()
-                f.readline()
-                proteins = []
-                for line in f:
-                    proteins.append(line.strip())
-        elif path == MSPInitializer.go_path:
-            name = file.replace(".txt", "")
-            with open(fullpath) as f:
-                proteins = []
-                for line in f:
-                    proteins.append(line.strip())
-        else:
-            raise ValueError(f"Invalid path: {path}")
+    def read_config_txt_file(self, file, is_pathway: bool = True) -> Tuple[str, list]:
+        path_full = os.path.join(self.start_dir, "pathways" if is_pathway else "go_terms", file)
+        if not os.path.isfile(path_full):
+            path_full = os.path.join(path_package_config, "pathways" if is_pathway else "go_terms", file)
+            if not os.path.isfile(path_full):
+                raise FileNotFoundError(f"The selected file: {file} cannot be found.")
+        name = file.replace(".txt", "")
+        with open(path_full) as f:
+            proteins = []
+            for line in f:
+                proteins.append(line.strip())
         return name, proteins
 
     def update_config_file(self):
@@ -185,6 +217,12 @@ class MSPInitializer:
         os.rename(yml_file_loc_tmp, yml_file_loc)
 
     def read_data(self):
+        """
+        | Initiates the file reader by providing the directory and the configs to the reader.
+        | The configs for the reader are taken from the configs from the name of the reader as key. E.g. mqreader.
+        | In turn a :class:`~mspypeline.helpers.Utils.DataDict` is generated to provide the mapping to the input data
+          (*reader_data*) for the further analysis with the :ref:`mspypeline plotters <plotters>`.
+        """
         for Reader in BaseReader.__subclasses__():
             Reader: Type[BaseReader]  # for IDE hints
             try:
@@ -196,6 +234,6 @@ class MSPInitializer:
                 self.logger.debug("No files found for reader: %s", Reader.name)
 
         # read all proteins and receptors of interest from the config dir
-        self.logger.info("Reading proteins and receptors of interest")
+        self.logger.info("Reading pathway and GO list of interest")
         self.interesting_proteins, self.go_analysis_gene_names = self.init_interest_from_txt()
         self.update_config_file()
