@@ -7,12 +7,39 @@ from typing import Optional, Iterable
 
 from mspypeline import create_app
 from mspypeline.core import MSPInitializer
+from mspypeline.helpers import get_logger
 from mspypeline.modules import default_normalizers
 from mspypeline.file_reader import BaseReader, MQReader
 
 
 class UIHandler:
-    def __init__(self, file_dir, yml_file=None, gui=False, host_flask=False, selected_reader=MQReader.MQReader, loglevel=logging.DEBUG, configs: dict = None):
+    """
+    | Used to take the mapping of arguments provided by the :class:`~MSPParser` through the command line to organize
+      through which point of entry the data analysis should be performed.
+    | The UIHandler can initialize the creation of the GUI, perform the whole analysis instantly by running the
+      pipeline and creating all plots according to the configs or (not yet available) host the mspypeline on a flask
+      server.
+    """
+    def __init__(self, file_dir, yml_file=None, gui=False, host_flask=False, selected_reader=MQReader.MQReader,
+                 loglevel=logging.DEBUG, configs: dict = None):
+        """
+        Parameters
+        ----------
+        file_dir
+            location where the directory/txt folder to the data can be found.
+        yml_file
+            path to the yaml config file
+        gui
+            should a GUI be compiled
+        host_flask
+            currently not implemented (should a flsk server be hosted)
+        selected_reader
+            instance of an :class:`~BaseReader` used to process data to an internal format for the plotter.
+        loglevel
+            level of the logger
+        configs
+            mapping of configuration
+        """
         base_config = {
             "has_techrep": False,
             "has_groups": False,
@@ -42,12 +69,29 @@ class UIHandler:
 
 
 class MSPGUI(tk.Tk):
+    """
+    | This class is called from the :class:`~UIHandler` to create the GUI.
+    """
     def __init__(self, file_dir, yml_file=None, loglevel=logging.DEBUG, configs: dict = None):
+        """
+        Parameters
+        ----------
+        file_dir
+            location where the directory/txt folder to the data can be found.
+        yml_file
+            path to the yaml config file
+        loglevel
+            level of the logger
+        configs
+            mapping containing the configurations
+        """
         super().__init__()
         self.yaml_options = ["default"]
         self.reader_options = {reader.name: reader for reader in BaseReader.__subclasses__()}
         self.selected_reader = MQReader.MQReader
         self.normalize_options = ["None"] + list(default_normalizers.keys())
+        self.mspinit = MSPInitializer(file_dir, yml_file, loglevel=loglevel)
+        self.logger = get_logger(self.__class__.__name__, loglevel=loglevel)
 
         self.number_of_plots = 0
 
@@ -87,18 +131,18 @@ class MSPGUI(tk.Tk):
 
         design_label = tk.Label(self, text="Replicate names").grid(row=3, column=2)
 
-        self.go_proteins_list = tk.Listbox(self, selectmode="multiple", height=5,
-                                           width=len(max(MSPInitializer.possible_gos, key=len)))
-        self.go_proteins_list.configure(exportselection=False)
-        for x in MSPInitializer.possible_gos:
-            self.go_proteins_list.insert("end", x)
+        self.go_term_list = tk.Listbox(self, selectmode="multiple", height=5,
+                                       width=len(max(self.mspinit.list_full_gos, key=len)))
+        self.go_term_list.configure(exportselection=False)
+        for x in self.mspinit.list_full_gos:
+            self.go_term_list.insert("end", x)
 
-        self.go_proteins_list.grid(row=4, column=0)
+        self.go_term_list.grid(row=4, column=0)
 
         self.pathway_list = tk.Listbox(self, selectmode="multiple", height=5,
-                                       width=len(max(MSPInitializer.possible_pathways, key=len)))
+                                       width=len(max(self.mspinit.list_full_pathways, key=len)))
         self.pathway_list.configure(exportselection=False)
-        for x in MSPInitializer.possible_pathways:
+        for x in self.mspinit.list_full_pathways:
             self.pathway_list.insert("end", x)
 
         self.pathway_list.grid(row=4, column=1)
@@ -175,13 +219,8 @@ class MSPGUI(tk.Tk):
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
 
-        # here we set the configurations
-        # constructing the class reads the yaml file once
-        self.mspinit = MSPInitializer(file_dir, yml_file, loglevel=loglevel)
-        if file_dir:
-            # the dir setting reads the yaml file twice, once after setting the dir, then after setting the yaml path
-            self.dir_text.set(file_dir)
         self.mspinit.configs.update(configs)
+        self.update_yaml_options()
         self.mainloop()
 
     def dir_setter(self, *args):
@@ -209,17 +248,23 @@ class MSPGUI(tk.Tk):
         for op in self.mspinit.configs.get(self.selected_reader.name, {}).get("all_replicates", []):
             self.experiments_list.insert("end", op)
         # clear selection then select from configs
-        for i, pathway in enumerate(MSPInitializer.possible_pathways):
+        for i, pathway in enumerate(self.mspinit.list_full_pathways):
             self.pathway_list.select_clear(i)
         if self.mspinit.configs.get("pathways"):
             for pathway in self.mspinit.configs.get("pathways"):
-                self.pathway_list.select_set(MSPInitializer.possible_pathways.index(pathway))
+                try:
+                    self.pathway_list.select_set(self.mspinit.list_full_pathways.index(pathway))
+                except ValueError:
+                    self.logger.warning("Selected pathway file %s not found", pathway)
         # clear selection then select from configs
-        for i, go in enumerate(MSPInitializer.possible_gos):
-            self.go_proteins_list.select_clear(i)
+        for i, go in enumerate(self.mspinit.list_full_gos):
+            self.go_term_list.select_clear(i)
         if self.mspinit.configs.get("go_terms"):
             for go in self.mspinit.configs.get("go_terms"):
-                self.go_proteins_list.select_set(MSPInitializer.possible_gos.index(go))
+                try:
+                    self.go_term_list.select_set(self.mspinit.list_full_gos.index(go))
+                except ValueError:
+                    self.logger.warning("Selected go term file %s not found", go)
 
     def yaml_path_setter(self, *args):
         self.mspinit.file_path_yaml = self.yaml_text.get()
@@ -274,11 +319,12 @@ class MSPGUI(tk.Tk):
             additional_settings = {k: v for k, v in additional_settings.items()
                                    if k != "create_plot" and k != "dfs_to_use" and k != "levels"}
             selected_settings.update(additional_settings)
-            self.mspinit.configs.update({plot_settings: selected_settings})
-        gos = self.go_proteins_list.curselection()
-        gos = [MSPInitializer.possible_gos[int(go)] for go in gos]
+            for k, v in selected_settings.items():
+                self.mspinit.configs[plot_settings][k] = v
+        gos = self.go_term_list.curselection()
+        gos = [self.mspinit.list_full_gos[int(go)] for go in gos]
         pathways = self.pathway_list.curselection()
-        pathways = [MSPInitializer.possible_pathways[int(pathway)] for pathway in pathways]
+        pathways = [self.mspinit.list_full_pathways[int(pathway)] for pathway in pathways]
         self.mspinit.configs["plot_r_volcano_settings"]["adj_pval"] = bool(self.p_val_var.get())
         self.mspinit.configs["go_terms"] = gos
         self.mspinit.configs["pathways"] = pathways
@@ -363,6 +409,10 @@ class MultiSelectOptionMenu(tk.Frame):
 
 
 class MSPParser(argparse.ArgumentParser):
+    """
+    | Uses the ``argparse`` module to provide a parser for command line options, arguments and sub-commands through
+      which the analysis can be started or the GUI can be called (see :ref:`get-started`).
+    """
     def __init__(self):
         super().__init__(description="A pipeline to analyze result files from a MaxQuant report. "
                                      "The required result files are in the txt directory.")
